@@ -1,98 +1,4 @@
-/* aumix.c: adjust /dev/mixer   copyright (c) 1993, 1996 the authors
-
-   AUTHORS:
-
-   - Savio Lam (versions 0.1 and 0.2)
-   - TimeCop (version 1.0)
-   - Trevor Johnson <trevor@jpj.net> (versions 1.1, 1.2, 1.3, 1.4, 1.5, 1.6, 
-   1.6.1)
-   - gpm code based on rmev.c by Alessandro Rubini <alessandro.rubini@linux.it>
-   - Kenn Humborg <kenn@wombat.ie> (version 1.4)
-   - Dan Fandrich <dan@fch.wimsey.bc.ca> (version 1.4)
-   - Jeff Spirko <spirko@topquark.physics.lehigh.edu> (version 1.5)
-
-   HISTORY:
-
-   1993-10-23 - version 0.1
-
-   1993-11-28 - version 0.2
-   - doesn't assume existence of any device (volume, bass, etc)
-   - some minor bug fixes
-   - some bug fixes to work correctly with SB Pro
-   - should now work correctly when multiple copies are run simutaneously
-
-   1993-11-28 - version 0.2
-
-   1996-08-23 - version 1.0
-   - added command-line interface
-
-   1996-09-02 - version 1.1
-   - fewer colors and no border to simplify full-screen interface
-   - knobs are highlighted as you move to them, then brighten and
-   leave a trail behind when you adjust them
-   - horizontal knobs drawn with same characters as vertical ones
-   - vertical controls spaced more closely so they fit in 80 columns
-   - use of "<" and ">" for cursor motion, and "q" to quit program
-   - use of tab, "+" and "-" keys shown on help line
-   - added "h" option for command line
-
-   1996-09-07 - version 1.2
-   - two banks of horizontal controls, no special treatment of
-   volume, bass, and treble (layout idea borrowed from CAM), so
-   labels don't have to be jammed together
-   - record/play indicators now show the letters "R" and "P"
-   rather than just a red or green square, so they should be
-   more self-explanatory and more monochrome-friendly
-   - added use of "[" and "]" keys to set controls to 0% or 100%
-
-   1996-09-13 - version 1.3
-   - added gpm mouse support by incorporating code from rmev.c by 
-   Alessandro Rubini (packaged with gpm) 
-   - added Quit button for use with mouse
-   - changed color of controls from white to cyan/green so the gpm 
-   cursor would be more visible
-   - added keys "Q" and ctrl-D to quit from program
-   - removed use of two different graphic characters to show controls in use 
-   or not in use
-
-   1996-12-02 - version 1.4
-   - added -b, -t, -s, and -m options (K.H. and D.F.)
-   - moved printf statements after the ioctl call, since this makes more
-   sense if the ioctl fails (K.H.) 
-   - changed layout to allow for balance controls, and added them (T.J.) 
-   - eliminated "level" array:  levels are now read by ioctl so changes by other
-   processes system-wide will be properly reflected (T.J.)
-   - conditional compilation:  GPM now optional, and interactive mode can be
-   disabled entirely (T.J.)
-
-   1996-12-06 - version 1.5
-   - "q" options for command line to print settings without changing (J.S.)
-   - command-line section modularized for compactness (T.J.) 
-   - widened controls in interactive mode for greater precision in setting and
-   reading (T.J.)
-   - balance controls get redrawn after levels adjusted (T.J.)
-   - added command-line options for imix, PCM2, reclev, igain, ogain, line1, 
-   line2 and line3 (T.J.) 
-
-   1996-12-15 - version 1.6
-   - ability to save settings to a file and load them in again
-   - option to use a restricted character set in interactive mode, when IBM
-   graphic characters are not available
-   - support for second mixer
-   - changed meaning of "<" and ">" keys (now same as Tab or Enter)
-   - "," and "." function synonymously with "<" and ">" so users need not shift 
-   when using US keyboard
-   - prettified device names removed, to save space and to allow for future
-   changes to the sound driver
-   - added labels for banks of level and balance controls
-   - fixed bug with record/play indicators
-   - record/play information updated more often
-   - record/play information gets printed on query
-
-   1996-12-17 - version 1.6.1
-   - fixed problem with conditional compilation
-
-   LICENSE:
+/* aumix.c: adjust /dev/mixer   copyright (c) 1993, 1996, 1997 the authors
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the Free
@@ -118,14 +24,17 @@
 #include <sys/soundcard.h>
 #ifdef GPM
 #include <gpm.h>
-#ifndef NCURSES
-#define NCURSES			/* How can you have a mouse if you don't have ncurses? */
-#endif
+#else
+#include "gpm-xterm.h"
 #endif
 #ifdef NCURSES
 #include <ncurses.h>
+#else
+#include <curses.h>
 #endif
 
+#define AUMIX_VERSION "1.8"
+#define SYSTEM_AUMIXRC "/etc/aumixrc"
 #ifdef NCURSES
 #define TITLE_COLOR 1
 #define TITLE_COLOR2 2
@@ -137,7 +46,7 @@
 #define INACTIVE_COLOR 8
 #define INCREMENT 4
 #define XOFFSET 9
-#define YOFFSET 2
+#define YOFFSET 0
 #else				/* these are defined in ncurses.h */
 #define TRUE 1
 #define FALSE 0
@@ -149,39 +58,47 @@ void Usage(void);
 void SetShowNoninter(int dev);
 void LoadSettings(void);
 void SaveSettings(void);
-#ifdef GPM
 int MouseHandler(Gpm_Event * event, void *data);
-#endif
 #ifdef NCURSES
 void InitColors(void);
 void DrawHandles(int dev);
 void Inter(void);
 void AdjustLevel(int dev, int incr, int setlevel);
 void SwitchRP(int dev);
+void DrawRP(int dev);
 void InitScreen(void);
 void AdjustBalance(int dev, int incr, int setlevel);
 void DrawLevelBalMode(int dev, int mode);
 void RedrawBalance(int dev);
 void EraseLevel(int dev);
+void CloseScreen(void);
+void AboutBox(void);
+void KeysBox(void);
+void ToggleMute(void);
 #endif
 
 #ifdef NCURSES
+WINDOW *w_about;
+WINDOW *w_keys;
 WINDOW *w_panel;
 #endif
 FILE *setfile;
 char setfile_name[256];
 short setfile_opened = FALSE, setfile_write_perm = FALSE, setfile_read_perm = FALSE;
 int devmask = 0, recmask = 0, recsrc = 0, stereodevs = 0, mixer_fd;
-int opened = FALSE, interactive = FALSE;
-int handle_char = 178, track_char = 193;
-char *devname[SOUND_MIXER_NRDEVICES] = SOUND_DEVICE_LABELS;
-char *optarg;
+int level[SOUND_MIXER_NRDEVICES];	/* store levels for muting */
+int mutestate = FALSE, opened = FALSE, interactive = FALSE;
+char *dev_name[SOUND_MIXER_NRDEVICES] = SOUND_DEVICE_NAMES;
+char *dev_label[SOUND_MIXER_NRDEVICES] = SOUND_DEVICE_LABELS;
+
+/* An index into this array gives the device number for the
+   corresponding command-line option. */
+char *moptindx = "vbtswplmcxWrio123";
+char *mopt;
 
 int main(int argc, char *const argv[], const char *optstring)
 {
-#ifdef GPM
     Gpm_Connect conn;
-#endif
     int optn, i;
     InitMixer(0);
     if (argc <= 1) {
@@ -201,88 +118,34 @@ int main(int argc, char *const argv[], const char *optstring)
 	optn = getopt(argc, argv, "dhLqSI:v:b:t:s:w:p:l:m:c:x:W:r:i:o:1:2:3:");
 	if (optn == -1)
 	    break;
-	switch (optn) {
-	case 'q':
-	    optarg = "qvbtswplmcxWrio123";
-	    for (i = 0; i < SOUND_MIXER_NRDEVICES; i++) {
-		if ((1 << i) & devmask)
-		    SetShowNoninter(i);
+	if ((mopt = strchr(moptindx, optn))) {
+	    SetShowNoninter(mopt - moptindx);
+	} else
+	    switch (optn) {
+	    case 'q':
+		optarg = "q";
+		for (i = 0; i < SOUND_MIXER_NRDEVICES; i++) {
+		    if ((1 << i) & devmask)
+			SetShowNoninter(i);
+		}
+		break;
+	    case 'd':
+		InitMixer(1);
+		break;
+	    case 'S':		/* save to file */
+		SaveSettings();
+		break;
+	    case 'L':		/* load from file */
+		LoadSettings();
+		break;
+	    case 'I':
+		interactive = TRUE;
+		break;
+	    default:
+		Usage();
 	    }
-	    break;
-/* to do: change over to use index() */
-	case 'v':
-	    SetShowNoninter(SOUND_MIXER_VOLUME);
-	    break;
-	case 'b':
-	    SetShowNoninter(SOUND_MIXER_BASS);
-	    break;
-	case 't':
-	    SetShowNoninter(SOUND_MIXER_TREBLE);
-	    break;
-	case 's':
-	    SetShowNoninter(SOUND_MIXER_SYNTH);
-	    break;
-	case 'w':
-	    SetShowNoninter(SOUND_MIXER_PCM);
-	    break;
-	case 'p':
-	    SetShowNoninter(SOUND_MIXER_SPEAKER);
-	    break;
-	case 'l':
-	    SetShowNoninter(SOUND_MIXER_LINE);
-	    break;
-	case 'm':
-	    SetShowNoninter(SOUND_MIXER_MIC);
-	    break;
-	case 'c':
-	    SetShowNoninter(SOUND_MIXER_CD);
-	    break;
-	case 'x':
-	    SetShowNoninter(SOUND_MIXER_IMIX);
-	    break;
-	case 'W':
-	    SetShowNoninter(SOUND_MIXER_ALTPCM);
-	    break;
-	case 'r':
-	    SetShowNoninter(SOUND_MIXER_RECLEV);
-	    break;
-	case 'i':
-	    SetShowNoninter(SOUND_MIXER_IGAIN);
-	    break;
-	case 'o':
-	    SetShowNoninter(SOUND_MIXER_OGAIN);
-	    break;
-	case '1':
-	    SetShowNoninter(SOUND_MIXER_LINE1);
-	    break;
-	case '2':
-	    SetShowNoninter(SOUND_MIXER_LINE2);
-	    break;
-	case '3':
-	    SetShowNoninter(SOUND_MIXER_LINE3);
-	    break;
-	case 'd':
-	    InitMixer(1);
-	    break;
-	case 'S':		/* save to file */
-	    SaveSettings();
-	    break;
-	case 'L':		/* load from file */
-	    LoadSettings();
-	    break;
-	case 'I':
-	    interactive = TRUE;
-	    if (*optarg == 'd') {
-/* user said we can't use IBM character set */
-		handle_char = 'O';
-		track_char = '=';
-	    }
-	    break;
-	default:
-	    Usage();
-	}
     }
-    if (interactive == FALSE)
+    if (!interactive)
 	exit(0);
 #ifdef NCURSES
     initscr();
@@ -290,35 +153,22 @@ int main(int argc, char *const argv[], const char *optstring)
     keypad(stdscr, TRUE);
     cbreak();
     noecho();
-    if (!has_colors())
-	fprintf(stderr, "aumix: warning: no color capability detected.\n");
     InitColors();
     InitScreen();
 #endif
-#ifdef GPM
     conn.eventMask = ~0;
     conn.defaultMask = GPM_MOVE | GPM_HARD;
     conn.maxMod = ~0;
     conn.minMod = 0;
-    if (Gpm_Open(&conn, 0) == -1) {
-	attrset(COLOR_PAIR(TITLE_COLOR) | A_BOLD | A_ALTCHARSET);
-	move(0, 35);
-	addstr("mouse off");
-	refresh();
-    }
+    i = Gpm_Open(&conn, 0);
     gpm_handler = MouseHandler;
-#endif
 #ifdef NCURSES
     Inter();
 #endif
-#ifdef GPM
     Gpm_Close();
-#endif
     close(mixer_fd);
 #ifdef NCURSES
-    clear();
-    refresh();
-    endwin();
+    CloseScreen();
     if (interactive) {
 	if (setfile_opened)
 	    fclose(setfile);
@@ -332,46 +182,40 @@ void LoadSettings(void)
 /* read settings from file */
     char tmpstring[80], recplay;
     int tmp, i, j = TRUE, left, right;
+    short try_etc = FALSE;
     if (getenv("HOME")) {
 	sprintf(setfile_name, "%s/.aumixrc", (char *) getenv("HOME"));
     } else {
-#ifdef NCURSES
-	if (interactive) {
-	    clear();
-	    refresh();
-	    endwin();
-	}
-#endif
-	printf("aumix: problem opening ~/.aumixrc: $HOME not set\n");
-	exit(1);
+	try_etc = TRUE;
+    }
+    if (!try_etc)
+	setfile = fopen(setfile_name, "r");
+    if (try_etc || !setfile) {
+/* couldn't open ~/.aumixrc, so try /etc/aumixrc */
+	sprintf(setfile_name, SYSTEM_AUMIXRC);
+	setfile = fopen(setfile_name, "r");
     }
-    setfile = fopen(setfile_name, "r");
     if (!setfile) {
 #ifdef NCURSES
-	if (interactive) {
-	    clear();
-	    refresh();
-	    endwin();
-	}
+	CloseScreen();
 #endif
-	perror("aumix: error opening .aumixrc");
+	perror("aumix: could not open $HOME/.aumixrc or /etc/aumixrc");
 	exit(1);
     }
-    for (j = TRUE; (!(j == EOF) && !(j == 0)); j = fscanf(setfile, "%5c:%3u:%3u:%1c\n", tmpstring, &left, &right, &recplay)) {
+    for (j = TRUE; (!(j == EOF) && !(j == 0)); j = fscanf(setfile, "%[^:]:%3u:%3u:%1c\n", tmpstring, &left, &right, &recplay)) {
 	for (i = 0; i < SOUND_MIXER_NRDEVICES; i++) {
 /* cycle through device names, looking for matches */
-	    if (!strncmp(tmpstring, devname[i], 5)) {
+	    if (!strcmp(tmpstring, dev_name[i])) {
 		left = (left > 100) ? 100 : left;
 		left = (left < 0) ? 0 : left;
 		right = (right > 100) ? 100 : right;
 		right = (right < 0) ? 0 : right;
 		tmp = left + (right << 8);
-		if (ioctl(mixer_fd, MIXER_WRITE(i), &tmp) == -1) {
-		    perror("MIXER_WRITE");
-		    exit(-1);
+		if ((ioctl(mixer_fd, MIXER_WRITE(i), &tmp) == -1) && !interactive) {
+		    printf("%s not set\n", dev_name[i]);
 		} else {
 		    if (!interactive)
-			printf("%s set to %i, %i", devname[i], left, right);
+			printf("%s set to %i, %i", dev_name[i], left, right);
 		    if ((1 << i) & recmask) {
 			recsrc = (recplay == 'R') ? recsrc | (1 << i) : recsrc &
 			    ~(1 << i);
@@ -404,11 +248,7 @@ void SaveSettings(void)
 	sprintf(setfile_name, "%s/.aumixrc", (char *) getenv("HOME"));
     } else {
 #ifdef NCURSES
-	if (interactive) {
-	    clear();
-	    refresh();
-	    endwin();
-	}
+	CloseScreen();
 #endif
 	printf("aumix: problem opening ~/.aumixrc: $HOME not set\n");
 	exit(1);
@@ -416,11 +256,7 @@ void SaveSettings(void)
     setfile = fopen(setfile_name, "w");
     if (!setfile) {
 #ifdef NCURSES
-	if (interactive) {
-	    clear();
-	    refresh();
-	    endwin();
-	}
+	CloseScreen();
 #endif
 	perror("aumix: error opening .aumixrc");
 	exit(1);
@@ -428,11 +264,7 @@ void SaveSettings(void)
     if (ioctl(mixer_fd, SOUND_MIXER_READ_RECSRC, &recsrc) == -1) {
 	perror("SOUND_MIXER_READ_RECSRC");
 #ifdef NCURSES
-	if (interactive) {
-	    clear();
-	    refresh();
-	    endwin();
-	}
+	CloseScreen();
 #endif
 	fclose(setfile);
 	exit(-1);
@@ -443,7 +275,7 @@ void SaveSettings(void)
 		perror("MIXER_READ");
 		exit(-1);
 	    }
-	    fprintf(setfile, "%s:%i:%i:%c\n", devname[i], (tmp & 0xFF), ((tmp >> 8) & 0xFF), ((1 << i) & recsrc ? 'R' : 'P'));
+	    fprintf(setfile, "%s:%i:%i:%c\n", dev_name[i], (tmp & 0xFF), ((tmp >> 8) & 0xFF), ((1 << i) & recsrc ? 'R' : 'P'));
 	}
     }
     fclose(setfile);
@@ -453,12 +285,12 @@ void InitMixer(int minor)
 {
     if (minor) {
 	if ((mixer_fd = open("/dev/mixer1", O_RDWR)) < 0) {
-	    fprintf(stderr, "Error opening /dev/mixer1.\n");
+	    fprintf(stderr, "aumix: error opening /dev/mixer1.\n");
 	    exit(1);
 	}
     } else {
 	if ((mixer_fd = open("/dev/mixer", O_RDWR)) < 0) {
-	    fprintf(stderr, "Error opening /dev/mixer.\n");
+	    fprintf(stderr, "aumix: error opening /dev/mixer.\n");
 	    exit(1);
 	}
     }
@@ -490,12 +322,12 @@ void SetShowNoninter(int dev)
     char *devstr;
     int tmp;
     if ((*optarg == 'q') || (*optarg == ' ')) {
-	devstr = devname[dev];
+	devstr = dev_name[dev];
 	if (ioctl(mixer_fd, MIXER_READ(dev), &tmp) == -1) {
 	    perror("MIXER_READ");
 	    exit(-1);
 	}
-	printf("%s %i, %i", devname[dev], (tmp & 0xFF), ((tmp >> 8) & 0xFF));
+	printf("%s %i, %i", dev_name[dev], (tmp & 0xFF), ((tmp >> 8) & 0xFF));
 	if ((1 << (dev)) & recmask) {
 	    if (ioctl(mixer_fd, SOUND_MIXER_READ_RECSRC, &recsrc) == -1) {
 		perror("SOUND_MIXER_READ_RECSRC");
@@ -521,51 +353,108 @@ void SetShowNoninter(int dev)
 void InitScreen()
 {
     int i;
-    move(0, 0);
-    attrset(COLOR_PAIR(TITLE_COLOR) | A_BOLD | A_ALTCHARSET);
-    printw("  uit  oad  ave         "
-	   "                                            aumix 1.6.1 ");
-    move(0, 1);
-    attrset(COLOR_PAIR(TITLE_COLOR2) | A_BOLD | A_ALTCHARSET);
-    printw("Q");
-    move(0, 6);
-    printw("L");
-    move(0, 11);
-    printw("S");
-    move(LINES - 1, 0);
-    addstr("Page Arrows ");
-    move(LINES - 1, 19);
-    addstr("Tab Enter < > ");
-    move(LINES - 1, 43);
-    addstr("+ - [ ] Arrows ");
-    move(LINES - 1, 65);
-    addstr("Space ");
-    attrset(COLOR_PAIR(TITLE_COLOR) | A_BOLD | A_ALTCHARSET);
-    move(LINES - 1, 12);
-    addstr("Device");
-    move(LINES - 1, 33);
-    addstr("Level/Bal");
-    move(LINES - 1, 58);
-    addstr("Adjust");
-    move(LINES - 1, 71);
-    addstr("Rec/Play");
-    refresh();
-    w_panel = newwin(LINES - 2, COLS, 1, 0);
-    wattrset(w_panel, COLOR_PAIR(INACTIVE_COLOR) | A_DIM | A_ALTCHARSET);
+    w_panel = newwin(LINES - 1, COLS, 0, 0);
     wclear(w_panel);
-    wmove(w_panel, YOFFSET - 1, XOFFSET + 1);
-    waddstr(w_panel, "0         Level        100       L        Balance         R");
-    RefreshAllSettings();
+    wattrset(w_panel, COLOR_PAIR(TITLE_COLOR));
+    wmove(w_panel, 2, 1);
+    waddstr(w_panel, "uit   ");
+    wmove(w_panel, 3, 1);
+    waddstr(w_panel, "oad   ");
+    wmove(w_panel, 4, 1);
+    waddstr(w_panel, "ave   ");
+    wmove(w_panel, 5, 1);
+    waddstr(w_panel, "eys   ");
+    wmove(w_panel, 6, 1);
+    waddstr(w_panel, "bout  ");
+    wmove(w_panel, 7, 1);
+    waddstr(w_panel, "efresh");
+    wmove(w_panel, 8, 1);
+    waddstr(w_panel, "ute   ");
+    wattrset(w_panel, COLOR_PAIR(TITLE_COLOR) | A_UNDERLINE);
+    wmove(w_panel, 0, 0);
+    waddstr(w_panel, " aumix ");
+    wattrset(w_panel, COLOR_PAIR(TITLE_COLOR2) | A_BOLD);
+    wmove(w_panel, 2, 0);
+    waddstr(w_panel, "Q");
+    wmove(w_panel, 3, 0);
+    waddstr(w_panel, "L");
+    wmove(w_panel, 4, 0);
+    waddstr(w_panel, "S");
+    wmove(w_panel, 5, 0);
+    waddstr(w_panel, "K");
+    wmove(w_panel, 6, 0);
+    waddstr(w_panel, "A");
+    wmove(w_panel, 7, 0);
+    waddstr(w_panel, "R");
+    wmove(w_panel, 8, 0);
+    waddstr(w_panel, "M");
+    wattrset(w_panel, COLOR_PAIR(INACTIVE_COLOR));
+    if (YOFFSET + SOUND_MIXER_NRDEVICES <= LINES) {
+	wmove(w_panel, YOFFSET + SOUND_MIXER_NRDEVICES, XOFFSET + 1);
+	waddstr(w_panel, "0         Level        100          L        Balance         R");
+    }
     for (i = 0; i < SOUND_MIXER_NRDEVICES; i++) {
 	if ((1 << i) & devmask) {
-	    wattrset(w_panel, COLOR_PAIR(INACTIVE_COLOR) | A_BOLD | A_ALTCHARSET);
+	    wattrset(w_panel, COLOR_PAIR(INACTIVE_COLOR) | A_BOLD);
 	} else
-	    wattrset(w_panel, COLOR_PAIR(INACTIVE_COLOR) | A_DIM | A_ALTCHARSET);
+	    wattrset(w_panel, COLOR_PAIR(INACTIVE_COLOR) | A_DIM);
 	/* draw control labels */
 	wmove(w_panel, YOFFSET + i, XOFFSET + 28);
-	waddstr(w_panel, devname[i]);
+	waddstr(w_panel, dev_label[i]);
     }
+    refresh();
     wrefresh(w_panel);
+    RefreshAllSettings();
+}
+
+void AboutBox(void)
+{
+    int key, temp;
+    w_about = newwin(LINES - 2, COLS, 0, 0);
+    wclear(w_about);
+    wattrset(w_about, COLOR_PAIR(INACTIVE_COLOR));
+    wmove(w_about, 2, 0);
+    wprintw(w_about, "aumix version %s\n", AUMIX_VERSION);
+    wprintw(w_about, "compiled for sound driver version %i\n", SOUND_VERSION);
+#ifdef OSS_GETVERSION
+    wprintw(w_about, "running under sound driver version %i\n", ioctl(mixer_fd, OSS_GETVERSION, temp));
+#else
+    wprintw(w_about, "cannot determine version of running sound driver\n");
+#endif
+    wprintw(w_about, "color%sdetected\n", (has_colors())? " " : " not ");
+    wprintw(w_about, "\nPress a key to resume.");
+    wrefresh(w_about);
+    key = Gpm_Getch();
+    InitScreen();
+}
+
+void KeysBox(void)
+{
+    int key;
+    w_keys = newwin(LINES, COLS, 0, 0);
+    wclear(w_keys);
+    wattrset(w_keys, COLOR_PAIR(TITLE_COLOR2));
+    wmove(w_keys, 2, 0);
+    waddstr(w_keys, "Page Arrows   \n");
+    waddstr(w_keys, "Tab Enter < > \n");
+    waddstr(w_keys, "+ - [ ] Arrows\n");
+    waddstr(w_keys, "Space         \n");
+    wattrset(w_keys, COLOR_PAIR(INACTIVE_COLOR));
+    wprintw(w_keys, "\nPress a key to resume.");
+    wmove(w_keys, 1, 0);
+    wprintw(w_keys, "Key            Function");
+    wattrset(w_keys, COLOR_PAIR(TITLE_COLOR) | A_BOLD);
+    wmove(w_keys, 2, 15);
+    waddstr(w_keys, "Device   \n");
+    wmove(w_keys, 3, 15);
+    waddstr(w_keys, "Level/Bal\n");
+    wmove(w_keys, 4, 15);
+    waddstr(w_keys, "Adjust   \n");
+    wmove(w_keys, 5, 15);
+    waddstr(w_keys, "Rec/Play \n");
+    wrefresh(w_keys);
+    key = Gpm_Getch();
+    InitScreen();
 }
 
 void RefreshAllSettings(void)
@@ -575,17 +464,47 @@ void RefreshAllSettings(void)
 	if ((1 << i) & devmask) {
 	    EraseLevel(i);
 	    RedrawBalance(i);
-	    wattrset(w_panel, COLOR_PAIR(HANDLE_COLOR) | A_BOLD | A_ALTCHARSET);
+	    wattrset(w_panel, COLOR_PAIR(HANDLE_COLOR));
 	    DrawHandles(i);
 	    /* print record/play indicators */
-	    if ((1 << (i)) & recmask) {
-		wattrset(w_panel, COLOR_PAIR(((1 << i) & recsrc ? REC_LED_COLOR
-			     : PLAY_LED_COLOR)) | A_BOLD | A_ALTCHARSET);
-		wmove(w_panel, YOFFSET + i, XOFFSET + 0);
-		waddch(w_panel, ((1 << i) & recsrc ? 'R' : 'P'));
+	    if ((1 << (i)) & recmask)
+		DrawRP(i);
+	}
+    }
+    wrefresh(w_panel);
+}
+
+void ToggleMute(void)
+{
+    int dev;
+    if (mutestate) {
+/* restore from array */
+	for (dev = 0; dev < SOUND_MIXER_NRDEVICES; dev++) {
+	    if (ioctl(mixer_fd, MIXER_WRITE(dev), &level[dev]) == -1) {
+		perror("MIXER_WRITE");
+		exit(-1);
+	    }
+	}
+    } else {
+/* store levels in array, set them to zero */
+	for (dev = 0; dev < SOUND_MIXER_NRDEVICES; dev++) {
+	    if ((1 << dev) & devmask) {
+		if (ioctl(mixer_fd, MIXER_READ(dev), &level[dev]) == -1) {
+		    perror("MIXER_READ");
+		    exit(-1);
+		}
+		if (ioctl(mixer_fd, MIXER_WRITE(dev), 0) == -1) {
+		    perror("MIXER_WRITE");
+		    exit(-1);
+		}
 	    }
 	}
     }
+    mutestate = !mutestate;
+    RefreshAllSettings();
+    wattrset(w_panel, COLOR_PAIR(INACTIVE_COLOR));
+    wmove(w_panel, 1, 1);
+    wprintw(w_panel, mutestate ? "muted" : "     ");
     wrefresh(w_panel);
 }
 
@@ -606,14 +525,15 @@ void DrawHandles(int dev)
 {
     int temp;
     if (ioctl(mixer_fd, MIXER_READ(dev), &temp) == -1) {
+	CloseScreen();
 	perror("MIXER_READ");
-	endwin();
 	exit(-1);
     }
     EraseLevel(dev);
-    wattrset(w_panel, COLOR_PAIR(HANDLE_COLOR) | A_BOLD | A_ALTCHARSET);
+    wattrset(w_panel, COLOR_PAIR(HANDLE_COLOR) | ((has_colors())? A_BOLD : A_REVERSE));
+    wmove(w_panel, YOFFSET + dev, XOFFSET + 1 + ((temp & 0x7F) + 2) / 4);
+    waddch(w_panel, 'O');
     wmove(w_panel, YOFFSET + dev, XOFFSET + 1 + ((temp & 0x7F) + 2) / 4);
-    waddch(w_panel, handle_char);
 }
 
 void Inter(void)
@@ -622,19 +542,14 @@ void Inter(void)
 /* find first existing device */
     for (dev = 0; !(devmask & (1 << dev)); dev++);
 /* highlight device label */
-    wattrset(w_panel, COLOR_PAIR(ACTIVE_COLOR) | A_BOLD | A_ALTCHARSET);
+    wattrset(w_panel, COLOR_PAIR(ACTIVE_COLOR) | ((has_colors())? A_BOLD : A_REVERSE));
     wmove(w_panel, YOFFSET + dev, XOFFSET + 28);
-    waddstr(w_panel, devname[dev]);
+    waddstr(w_panel, dev_label[dev]);
     DrawHandles(dev);
     DrawLevelBalMode(dev, 0);
     wrefresh(w_panel);
     levelbalmode = 0;
-#ifdef GPM
     while ((key = Gpm_Getch()) != EOF) {
-#else
-    while (1) {
-	key = getch();
-#endif
 	incr = 0;
 	dir = 0;
 	switch (key) {
@@ -645,12 +560,12 @@ void Inter(void)
 	case 9:		/* tab */
 	case 10:		/* enter */
 	    levelbalmode = !levelbalmode;
-	    wattrset(w_panel, COLOR_PAIR(INACTIVE_COLOR) | A_BOLD | A_ALTCHARSET);
+	    wattrset(w_panel, COLOR_PAIR(INACTIVE_COLOR));
 	    wmove(w_panel, YOFFSET + dev, XOFFSET + 27);
 	    waddstr(w_panel, "       ");
-	    wattrset(w_panel, COLOR_PAIR(ACTIVE_COLOR) | A_BOLD | A_ALTCHARSET);
+	    wattrset(w_panel, COLOR_PAIR(ACTIVE_COLOR) | ((has_colors())? A_BOLD : A_REVERSE));
 	    wmove(w_panel, YOFFSET + dev, XOFFSET + 28);
-	    waddstr(w_panel, devname[dev]);
+	    waddstr(w_panel, dev_label[dev]);
 	    DrawLevelBalMode(dev, levelbalmode);
 	    break;
 	case 27:		/* Esc */
@@ -668,11 +583,12 @@ void Inter(void)
 		dir = 1;
 	    /* un-highlight current label */
 	    DrawHandles(dev);
-	    wattrset(w_panel, COLOR_PAIR(INACTIVE_COLOR) | A_BOLD | A_ALTCHARSET);
+	    wattrset(w_panel, COLOR_PAIR(INACTIVE_COLOR));
 	    wmove(w_panel, YOFFSET + dev, XOFFSET + 27);
 	    waddstr(w_panel, "       ");
+	    wattrset(w_panel, COLOR_PAIR(INACTIVE_COLOR) | A_BOLD);
 	    wmove(w_panel, YOFFSET + dev, XOFFSET + 28);
-	    waddstr(w_panel, devname[dev]);
+	    waddstr(w_panel, dev_label[dev]);
 /* switch to next existing device */
 	    do {
 		if (dir == 1) {
@@ -686,11 +602,10 @@ void Inter(void)
 		}
 	    }
 	    while (!((1 << dev) & devmask));
-
 	    /* highlight new device label */
-	    wattrset(w_panel, COLOR_PAIR(ACTIVE_COLOR) | A_BOLD | A_ALTCHARSET);
+	    wattrset(w_panel, COLOR_PAIR(ACTIVE_COLOR) | ((has_colors())? A_BOLD : A_REVERSE));
 	    wmove(w_panel, YOFFSET + dev, XOFFSET + 28);
-	    waddstr(w_panel, devname[dev]);
+	    waddstr(w_panel, dev_label[dev]);
 	    DrawHandles(dev);
 	    DrawLevelBalMode(dev, levelbalmode);
 	    break;
@@ -721,14 +636,30 @@ void Inter(void)
 	    else
 		AdjustLevel(dev, incr, -1);
 	    break;
+	case 'K':
+	case 'k':
+	    KeysBox();
+	    break;
 	case 'L':
 	case 'l':
 	    LoadSettings();
 	    break;
+	case 'R':
+	case 'r':
+	    RefreshAllSettings();
+	    break;
 	case 'S':
 	case 's':
 	    SaveSettings();
 	    break;
+	case 'A':
+	case 'a':
+	    AboutBox();
+	    break;
+	case 'M':
+	case 'm':
+	    ToggleMute();
+	    break;
 	}
 	wrefresh(w_panel);
     }
@@ -737,13 +668,15 @@ void Inter(void)
 void DrawLevelBalMode(int dev, int mode)
 /* arrow to show whether keyboard commands will adjust level or balance */
 {
-    wattrset(w_panel, COLOR_PAIR(ACTIVE_COLOR) | A_BOLD | A_ALTCHARSET);
+    wattrset(w_panel, COLOR_PAIR(ACTIVE_COLOR) | ((has_colors())? A_BOLD : A_REVERSE));
     if ((mode) && ((1 << dev) & stereodevs)) {
 	wmove(w_panel, YOFFSET + dev, XOFFSET + 33);
 	waddch(w_panel, '>');
+	wmove(w_panel, YOFFSET + dev, XOFFSET + 33);
     } else {
 	wmove(w_panel, YOFFSET + dev, XOFFSET + 27);
 	waddch(w_panel, '<');
+	wmove(w_panel, YOFFSET + dev, XOFFSET + 27);
     }
 }
 
@@ -755,56 +688,79 @@ void AdjustLevel(int dev, int incr, int setlevel)
  *  setlevel: level to set directly, or -1 to increment
  */
 {
-    int temp;
+    int balset, max, left, right, temp, left_orig, right_orig;
+    if (!((1 << dev) & devmask) || (dev > SOUND_MIXER_NRDEVICES - 1) || (dev < 0))
+	return;
+    if (ioctl(mixer_fd, MIXER_READ(dev), &temp) == -1) {
+	CloseScreen();
+	perror("MIXER_READ");
+	exit(-1);
+    }
+    left = temp & 0x7F;
+    right = (temp >> 8) & 0x7F;
+    left_orig = left;
+    right_orig = right;
+    max = (left > right) ? left : right;
+    if (max) {
+	balset = (left > right) ? 50 * right / max : 100 - (50 * left / max);
+    } else {
+	balset = 50;
+    }
     if ((1 << dev) & devmask) {
 	if (dev > SOUND_MIXER_NRDEVICES - 1) {
 	    dev = SOUND_MIXER_NRDEVICES - 1;
 	} else {
 	    dev = (dev < 0) ? 0 : dev;
 	}
-	if (ioctl(mixer_fd, MIXER_READ(dev), &temp) == -1) {
-	    perror("MIXER_READ");
-	    endwin();
-	    exit(-1);
-	}
-	if ((1 << dev) & stereodevs) {
-	    /* Take the average of left and right settings.
-	       This messes up the balance setting. */
-	    temp = ((temp & 0x7F) + ((temp >> 8) & 0x7F)) / 2;
+	max += incr / 2;
+	if (balset > 49) {
+	    left = max * (100 - balset) / 50;
+	    right = max;
 	} else {
-	    temp = (temp & 0x7F);
+	    right = max * balset / 50;
+	    left = max;
 	}
-	temp += incr;
-	temp = (setlevel > -1) ? setlevel : temp;
-	temp = (temp > 100) ? 100 : temp;
-	temp = (temp < 0) ? 0 : temp;
-	temp |= (temp << 8);
-	if (ioctl(mixer_fd, MIXER_WRITE(dev), &temp) == -1) {
-	    perror("MIXER_WRITE");
-	    endwin();
-	    exit(-1);
+	left = (left > 100) ? 100 : left;
+	right = (right > 100) ? 100 : right;
+	left = (left < 0) ? 0 : left;
+	right = (right < 0) ? 0 : right;
+	if ((left == left_orig) && (right == right_orig)) {
+	    left += incr / 2;
+	    right += incr / 2;
 	}
-	EraseLevel(dev);
-	/* Draw handle at new position */
-	wattrset(w_panel, COLOR_PAIR(HANDLE_COLOR) | A_BOLD | A_ALTCHARSET);
-	wmove(w_panel, YOFFSET + dev, XOFFSET + 1 + ((temp & 0x7F) + 2) / 4);
-	waddch(w_panel, handle_char);
-	RedrawBalance(dev);
-	wrefresh(w_panel);
     }
+    left = (left > 100) ? 100 : left;
+    right = (right > 100) ? 100 : right;
+    left = (left < 0) ? 0 : left;
+    right = (right < 0) ? 0 : right;
+    left = (setlevel > -1) ? setlevel : left;
+    right = (setlevel > -1) ? setlevel : right;
+    temp = (right << 8) | left;
+    if (ioctl(mixer_fd, MIXER_WRITE(dev), &temp) == -1) {
+	CloseScreen();
+	perror("MIXER_WRITE");
+	exit(-1);
+    }
+    EraseLevel(dev);
+    /* Draw handle at new position */
+    RedrawBalance(dev);
+    wattrset(w_panel, COLOR_PAIR(HANDLE_COLOR) | ((has_colors())? A_BOLD : A_REVERSE));
+    wmove(w_panel, YOFFSET + dev, XOFFSET + 1 + ((((temp & 0x7F) + ((temp >> 8) & 0x7F)) / 2) + 2) / 4);
+    waddch(w_panel, 'O');
+    wmove(w_panel, YOFFSET + dev, XOFFSET + 1 + ((((temp & 0x7F) + ((temp >> 8) & 0x7F)) / 2) + 2) / 4);
+    wrefresh(w_panel);
 }
 
 void EraseLevel(int dev)
 {
 /* redraw level track */
     int i;
-    wattrset(w_panel, COLOR_PAIR(TRACK_COLOR) | A_BOLD | A_ALTCHARSET);
+    wattrset(w_panel, COLOR_PAIR(TRACK_COLOR));
     for (i = 0; i < 26; i++) {
 	wmove(w_panel, YOFFSET + dev, XOFFSET + 1 + i);
-	waddch(w_panel, track_char);
+	waddch(w_panel, '+');
     }
 }
-
 void AdjustBalance(int dev, int incr, int setabs)
 /*
    *  dev: device to adjust
@@ -818,8 +774,8 @@ void AdjustBalance(int dev, int incr, int setabs)
     if (((1 << dev) & devmask) & (dev > SOUND_MIXER_NRDEVICES - 1))
 	dev = SOUND_MIXER_NRDEVICES - 1;
     if (ioctl(mixer_fd, MIXER_READ(dev), &temp) == -1) {
+	CloseScreen();
 	perror("MIXER_READ");
-	endwin();
 	exit(-1);
     }
     left = temp & 0x7F;
@@ -860,8 +816,8 @@ void AdjustBalance(int dev, int incr, int setabs)
     }
     temp = (right << 8) | left;
     if (ioctl(mixer_fd, MIXER_WRITE(dev), &temp) == -1) {
+	CloseScreen();
 	perror("MIXER_WRITE");
-	endwin();
 	exit(-1);
     }
     /* draw handle at new position */
@@ -873,14 +829,14 @@ void RedrawBalance(int dev)
 {
     int left, right, max, temp, balset;
     if ((1 << dev) & stereodevs) {
-	wattrset(w_panel, COLOR_PAIR(TRACK_COLOR) | A_BOLD | A_ALTCHARSET);
+	wattrset(w_panel, COLOR_PAIR(TRACK_COLOR));
 	for (temp = 0; temp < 26; temp++) {
-	    wmove(w_panel, YOFFSET + dev, XOFFSET + 34 + temp);
-	    waddch(w_panel, track_char);
+	    wmove(w_panel, YOFFSET + dev, XOFFSET + 37 + temp);
+	    waddch(w_panel, '+');
 	}
 	if (ioctl(mixer_fd, MIXER_READ(dev), &temp) == -1) {
+	    CloseScreen();
 	    perror("MIXER_READ");
-	    endwin();
 	    exit(-1);
 	}
 	left = temp & 0x7F;
@@ -891,9 +847,10 @@ void RedrawBalance(int dev)
 	} else {
 	    balset = 50;
 	}
-	wattrset(w_panel, COLOR_PAIR(HANDLE_COLOR) | A_BOLD | A_ALTCHARSET);
-	wmove(w_panel, YOFFSET + dev, XOFFSET + 34 + balset / 4);
-	waddch(w_panel, handle_char);
+	wattrset(w_panel, COLOR_PAIR(HANDLE_COLOR) | ((has_colors())? A_BOLD : A_REVERSE));
+	wmove(w_panel, YOFFSET + dev, XOFFSET + 37 + balset / 4);
+	waddch(w_panel, 'O');
+	wmove(w_panel, YOFFSET + dev, XOFFSET + 37 + balset / 4);
 	wrefresh(w_panel);
     }
 }
@@ -911,49 +868,83 @@ void SwitchRP(int dev)
 	else
 	    recsrc |= (1 << dev);	/* turn on recording */
 	if (ioctl(mixer_fd, SOUND_MIXER_WRITE_RECSRC, &recsrc) == -1) {
+	    CloseScreen();
 	    perror("SOUND_MIXER_WRITE_RECSRC");
-	    endwin();
 	    exit(-1);
 	}
 	if (ioctl(mixer_fd, SOUND_MIXER_READ_RECSRC, &recsrc) == -1) {
 	    perror("SOUND_MIXER_READ_RECSRC");
 	    exit(-1);
 	}
-	wattrset(w_panel, COLOR_PAIR(((1 << dev) & recsrc
-	     ? REC_LED_COLOR : PLAY_LED_COLOR)) | A_BOLD | A_ALTCHARSET);
-	wmove(w_panel, YOFFSET + dev, XOFFSET + 0);
-	waddch(w_panel, ((1 << dev) & recsrc ? 'R' : 'P'));
-	wrefresh(w_panel);
+	DrawRP(dev);
+    }
+}
+
+void DrawRP(int dev)
+{
+    wattrset(w_panel,
+	     (1 << dev) & recsrc ? COLOR_PAIR(REC_LED_COLOR) | ((has_colors())? A_NORMAL : A_REVERSE)
+	     : COLOR_PAIR(PLAY_LED_COLOR)
+	);
+    wmove(w_panel, YOFFSET + dev, XOFFSET + 0);
+    waddch(w_panel, ((1 << dev) & recsrc ? 'R' : 'P'));
+    wmove(w_panel, YOFFSET + dev, XOFFSET + 0);
+    wrefresh(w_panel);
+}
+
+void CloseScreen(void)
+{
+    if (interactive) {
+	clear();
+	refresh();
+	endwin();
     }
 }
 #endif				/* end of ncurses stuff */
-
-#ifdef GPM
 int MouseHandler(Gpm_Event * event, void *data)
 {
     int dev, temp;
 /* quit */
-    if ((event->y == 1) & (event->x == 2) && (event->type & GPM_DOWN)) {
+    if ((event->y == 3) & (event->x < 8) && (event->type & GPM_DOWN)) {
 	Gpm_Close();
 	close(mixer_fd);
-	clear();
-	refresh();
-	endwin();
+	CloseScreen();
 	exit(0);
     }
-/* load from file */ if ((event->y == 1) & (event->x == 7) && (event->type & GPM_DOWN)) {
+/* load from file */
+    if ((event->y == 4) & (event->x < 8) && (event->type & GPM_DOWN)) {
 	LoadSettings();
 	return 0;
     }
 /* save to file */
-    if ((event->y == 1) & (event->x == 12) && (event->type & GPM_DOWN)) {
+    if ((event->y == 5) & (event->x < 8) && (event->type & GPM_DOWN)) {
 	SaveSettings();
 	return 0;
     }
-    dev = event->y - 2 - YOFFSET;
+/* help on keystrokes */
+    if ((event->y == 6) & (event->x < 8) && (event->type & GPM_DOWN)) {
+	KeysBox();
+	return 0;
+    }
+/* about */
+    if ((event->y == 7) & (event->x < 8) && (event->type & GPM_DOWN)) {
+	AboutBox();
+	return 0;
+    }
+/* refresh */
+    if ((event->y == 8) & (event->x < 8) && (event->type & GPM_DOWN)) {
+	RefreshAllSettings();
+	return 0;
+    }
+/* mute */
+    if ((event->y == 9) & (event->x < 8) && (event->type & GPM_DOWN)) {
+	ToggleMute();
+	return 0;
+    }
+    dev = event->y - 1 - YOFFSET;
     temp = event->x - 2 - XOFFSET;
-    if ((temp > 32) & (temp < 59)) {
-	temp -= 33;
+    if ((temp > 35) & (temp < 62)) {
+	temp -= 36;
 	temp *= 4;
 	if (((event->type & GPM_DOWN) || (event->type & GPM_DRAG)) && ((1 << dev) & stereodevs)) {
 	    AdjustBalance(dev, 0, temp);
@@ -973,29 +964,28 @@ int MouseHandler(Gpm_Event * event, void *data)
     }
     return 0;
 }
-#endif				/* end of GPM stuff */
 
 void Usage(void)
 {
-    printf("aumix 1.6.1 usage: aumix [[ -<device option>[ ]<level>|q[uery],\n");
-    printf("[ -<device option>[ ]<level>|q[uery], [...]][-d] [-h] [-I{d|c}] [-L] [-q] [-S]\n\n");
-    printf("device options:\n");
-    printf("  v: main Volume           x: miX monitor\n");
-    printf("  b: Bass                  W: PCM2\n");
-    printf("  t: Treble                r: Record\n");
-    printf("  s: Synth                 i: Input gain\n");
-    printf("  w: PCM                   o: Output gain\n");
-    printf("  p: PC speaker            1: line1\n");
-    printf("  l: Line                  2: line2\n");
-    printf("  m: Mic                   3: line3\n");
-    printf("  c: CD\n\n");
-    printf("other options:\n");
-    printf("   d:  Dual mixers; adjust secondary\n");
-    printf("   h:  this Helpful message\n");
-    printf("  Id:  start in Interactive mode; Don't use IBM graphic characters\n");
-    printf("  Ic:  start in Interactive mode after doing non-interactive functions\n");
-    printf("   L:  Load settings from ~/.aumixrc\n");
-    printf("   q:  Query all devices and print their settings\n");
-    printf("   S:  Save settings to ~/.aumixrc\n");
+    printf("aumix %s usage: aumix [[ -<device option>[ ]<level>|q[uery],\n", AUMIX_VERSION);
+    fputs(("\
+[ -<device option>[ ]<level>|q[uery], [...]][-d] [-h] [-I{d|c}] [-L] [-q] [-S]\n\n\
+device options:\n\
+  v: main volume           x: mix monitor\n\
+  b: bass                  W: PCM2\n\
+  t: treble                r: record\n\
+  s: synth                 i: input gain\n\
+  w: PCM                   o: output gain\n\
+  p: PC speaker            1: line1\n\
+  l: line                  2: line2\n\
+  m: mic                   3: line3\n\
+  c: CD\n\n\
+other options:\n\
+   d:  dual mixers; adjust secondary\n\
+   h:  this helpful message\n\
+   I:  start in interactive mode after doing non-interactive functions\n\
+   L:  load settings\n\
+   q:  query all devices and print their settings\n\
+   S:  save settings to ~/.aumixrc\n"), stdout);
     exit(1);
 }
